//------------------------------------------------------------------------------------------------
//
//   Greg's Mod Base - Generic Common Proxy
//
//------------------------------------------------------------------------------------------------

package gcewing.sg;

import java.util.*;
import java.lang.reflect.*;

import net.minecraft.entity.player.*;
import net.minecraft.inventory.*;
import net.minecraft.world.*;

import net.minecraftforge.common.*;
import cpw.mods.fml.common.network.*;

public class BaseProxy implements IGuiHandler {

	BaseMod base;

	Map<Integer, Class<? extends Container>> containerClasses =
		new HashMap<Integer, Class<? extends Container>>();

	public void init(BaseMod mod) {
		base = mod;
		registerContainers();
	}
	
	void registerContainers() {
	}
	
	public void addContainer(Enum id, Class<? extends Container> cls) {
		addContainer(id.ordinal(), cls);
	}

	public void addContainer(int id, Class<? extends Container> cls) {
		containerClasses.put(id, cls);
	}
	
	/**
	 * Returns a Container to be displayed to the user. 
	 * On the client side, this needs to return a instance of GuiScreen
	 * On the server side, this needs to return a instance of Container
	 *
	 * @param ID The Gui ID Number
	 * @param player The player viewing the Gui
	 * @param world The current world
	 * @param x X Position
	 * @param y Y Position
	 * @param z Z Position
	 * @return A GuiScreen/Container to be displayed to the user, null if none.
	 */

	@Override
	public Object getServerGuiElement(int id, EntityPlayer player, World world, int x, int y, int z) {
		Class cls = containerClasses.get(id);
		if (cls != null)
			return createGuiElement(cls, player, world, x, y, z);
		else
			return getGuiContainer(id, player, world, x, y, z);
	}
	
	Object createGuiElement(Class cls, EntityPlayer player, World world, int x, int y, int z) {
		try {
			Method creator;
			try {
				return cls.getMethod("create", EntityPlayer.class, World.class, int.class, int.class, int.class)
					.invoke(null, player, world, x, y, z);
			}
			catch (NoSuchMethodException e) {
				return cls.getConstructor(EntityPlayer.class, World.class, int.class, int.class, int.class)
					.newInstance(player, world, x, y, z);
			}
		}
		catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	Container getGuiContainer(int id, EntityPlayer player, World world, int x, int y, int z) {
		return null;
	}

	@Override
	public Object getClientGuiElement(int id, EntityPlayer player, World world, int x, int y, int z) {
		return null;
	}
	
//	public void printChatMessage(String s) {
//	}

}
